
#ifndef MP_PATH_H

#define MP_PATH_H 1

# include "mplib.h"
# include "mplibps.h"

/*
  mp_create_knot() allocates and returns a new knot.
  Returns NULL on (malloc) failure. 
 */

extern mp_knot * mp_create_knot (MP mp);

/*
  mp_set_knot() fills in the coordinate of knot |p|. 
  |fabs(x)| and |fabs(y)| should be less than 4096.0.
  return 1 on success, 0 on failure.
 */

extern int mp_set_knot (MP mp, mp_knot *p, double x, double y);

/*
  mp_final_knotpair() connects |p| and |q| using an 'endpoint join'. 
  Because all knot lists are cyclic, this is needed to end the path 
  properly (or use mp_cycle_knotpair()).
  return 1 on success, 0 on failure.
*/

extern int mp_final_knotpair (MP mp, mp_knot *p, mp_knot *q);

/*
  mp_append_knot() appends a knot to previous knot |q|, and returns the new knot. 
  This is a convenience method combining mp_create_knot(), mp_set_knot(), and
  (if |q| is not NULL) mp_final_knotpair(). 
  Returns NULL on failure. 
 */

extern mp_knot * mp_append_knot (MP mp, mp_knot *q, double x, double y);

/*
  mp_cycle_knotpair() connects |p| and |q| using an 'open join'. 
  This is needed to mimic metapost's 'cycle'.
  return 1 on success, 0 on failure.
*/

extern int mp_cycle_knotpair (MP mp, mp_knot *p, mp_knot *q);

/* 
   mp_set_knot_curl() sets the curl value for a knot. 
   |fabs(value)| should be less than 4096.0
   return 1 on success, 0 on failure.
*/

extern int mp_set_knot_curl (MP mp, mp_knot *q, 
			     double value);

/* 
   mp_set_knot_direction() sets the direction {x,y} value for a knot. 
   |fabs(x)| and |fabs(y)| should be less than 4096.0
   return 1 on success, 0 on failure.
*/

extern int mp_set_knot_direction (MP mp, mp_knot *q, 
				  double x, double y);

/* 
   mp_set_knotpair_tensions() sets the tension specifiers for a pair of
   connected knots.
   |fabs(t1)| and |fabs(t2)| should be more than 0.75 and less than 4096.0
   return 1 on success, 0 on failure.
*/

extern int mp_set_knotpair_tensions (MP mp, mp_knot *p, mp_knot *q, 
				 double t1, double t2);


/* 
   mp_set_knotpair_controls() sets explicit controls for a knotpair
   |fabs(x1)| and |fabs(y1)| should be less than 4096.0
   |fabs(x2)| and |fabs(y2)| should be less than 4096.0
   return 1 on success, 0 on failure.
*/

extern int mp_set_knotpair_controls (MP mp, mp_knot *p, mp_knot *q, 
				     double x1, double y1, double x2, double y2);

/* 
   mp_set_knotpair_curls() is a convenience method that calls 
   mp_set_knot_curl(mp,p,t1) and mp_set_knot_curl(mp,q,t2)
   return 1 if both succeed, 0 otherwise.
*/
extern int mp_set_knotpair_curls (MP mp, mp_knot *p, mp_knot *q, 
			      double t1, double t2);

/* 
   mp_set_knotpair_directions() is a convenience method that calls 
   mp_set_knot_direction(mp,p,x1,y1) and mp_set_knot_direction(mp,p,x2,y2)
   return 1 if both succeed, 0 otherwise.
*/

extern int mp_set_knotpair_directions (MP mp, mp_knot *p, mp_knot *q, 
				   double x1, double y1, double x2, double y2);

/* 
   mp_solve_path() finds explicit controls for the knot list at |first|, which
   is changed in-situ. Returns 0 if there was any kind of error, in which
   case |first| is unmodified. There can be quite a set of potential errors,
   mostly harmless processing errors.
   However, be aware that it is also possible that there are internal mplib 
   memory allocation errors. A distinction between those can not be made 
   at the moment.
   return 1 on success, 0 on failure
 */

extern int mp_solve_path (MP mp, mp_knot *first);

/* 
   mp_dump_path() dumps the knot list starting at |h| on stdout using 
   a format similar to metapost itself.
*/

extern void mp_dump_path (MP mp, mp_knot *h);

/* 
   mp_free_path() frees the knot list starting at |h|.
*/

extern void mp_free_path (MP mp, mp_knot *h);

#endif

